/***************************************************************************
* IRCClient.java
*
* Copyright (c) 2008, kfmes & lindol All rights reserved.
* 
* kfmes(KIM Ga-hyeon, klchat@kfmes.com)

* Lindol(PARK Ju-Yeon, lindolsang@gmail.com) 
* 
* http://kldp.net/projects/klchat or http://klchat.kldp.net
* 
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * Neither the name of the klChat nor the
*       names of its contributors may be used to endorse or promote products
*       derived from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY kfmes & lindol ``AS IS'' AND ANY
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED. IN NO EVENT SHALL kfmes & lindol BE LIABLE FOR ANY
* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 **************************************************************************/
package kfmes.klBot;

import java.util.HashMap;

import org.schwering.irc.lib.IRCConnection;
import org.schwering.irc.lib.IRCEventListener;
import org.schwering.irc.lib.IRCModeParser;
import org.schwering.irc.lib.IRCUser;

/**
 * @author kfmes
 * @version $Id: IRCClient.java,v 1.6 2008/09/08 11:57:08 kfmes Exp $
 */
public class IRCClient {
	private IRCConnection conn = null;
	private HashMap<String, Integer> warningCount = new HashMap<String, Integer>();
	private HashMap<String, Integer> noticeCount = new HashMap<String, Integer>();
	private HashMap<String, Integer> scriptCount = new HashMap<String, Integer>();
	//private IRCEventListener listener = null; 
	private String bbangID = null;
	
	/**
	 * TODO:
	 * ACTION 처리
	 * 일정시간후에 reset
	 * 
	 * 
	 */
	
	public IRCClient(IRCConnection ircc) {
		this.conn =ircc;
		conn.addIRCEventListener(listener);
		
	}
	
	private void procCommand(String target, String msg){
		LogUtil.println(msg);
		String ll[] = msg.split(" ");
		if(msg.startsWith("!join")){
			conn.doJoin(ll[1]);
		}else if(msg.startsWith("!exit")){
			conn.doQuit();
			System.exit(0);
		}
		else if(msg.startsWith("!say"))
			conn.doPrivmsg(ll[1], msg.substring( msg.indexOf(ll[2])));
		else if(msg.startsWith("!!")){
			conn.send(msg.substring(  msg.indexOf(ll[1])));
		}
				
		
		else if(msg.startsWith("!^^")){
			conn.doPrivmsg("^^" , "인증 " + ll[1] + " " + ll[2]);
			this.bbangID = ll[1];
		}
		else if(msg.startsWith("!+x")){
			if(bbangID!=null)
				conn.send("mode " + conn.getNick() +   " +x\r\n");
		}
		else if(msg.startsWith("!part"))
			conn.doPart(ll[1]);
		else if(msg.startsWith("!nick"))
			conn.doNick(ll[1]);
		
	}
	private void procColorMessage(String target, IRCUser user, String msg){
		Integer count = warningCount.get(user.getNick());
		if(count==null)
			count = 0;
		warningCount.put(user.getNick(), ++count);
		
		if(count<5 + 1){
			conn.doPrivmsg(target, user.getNick() + "님 채널에서 색글 사용은 자제해 주시길 바랍니다.");
			LogUtil.fprintln("Warn(Color) : " + user.getNick() + "/" + msg);
		}
		else{
			String ban = "ban " + target + " "  + user.getNick() + " 3 99 색글금지";
			conn.doPrivmsg("^^", ban);
			LogUtil.fprintln(DateUtil.toTextString() +" " + ban);
			LogUtil.fprintln("Ban (Color) : " + user.getNick() + "/" + msg);
			warningCount.remove(user.getNick());
			}
	}
	
	
	private void procActionMessage(String target, IRCUser user, String msg){
		Integer count = warningCount.get(user.getNick());
		if(count==null)
			count = 0;
		warningCount.put(user.getNick(), ++count);
		
		if(count<10 ){
			conn.doPrivmsg(target, user.getNick() + "님 채널에서 액션메시지 사용은 자제해 주시길 바랍니다.");
			LogUtil.fprintln("Warn(AMsg): " + user.getNick() + "/" + msg);
		}
		else{
			String ban = "ban " + target + " "  + user.getNick() + " 3 99 액션메세지금지";
			conn.doPrivmsg("^^", ban);
			LogUtil.fprintln(DateUtil.toTextString() +" " + ban);
			LogUtil.fprintln("Ban (AMsg) : " + user.getNick() + "/" + msg);
			warningCount.remove(user.getNick());
			}
	}
	
	private void procScriptCount(String target, IRCUser user, String msg){
		Integer count = scriptCount.get(user.getNick());
		if(count==null)
			count = 0;
		warningCount.put(user.getNick(), ++count);
		
		if(count<7 ){
			conn.doPrivmsg(target, user.getNick() + "님 채널에서 과도한 스크립트 사용은 자제해 주시길 바랍니다.");
			LogUtil.fprintln("Warn(AMsg): " + user.getNick() + "/" + msg);
		}
		else{
			String ban = "ban " + target + " "  + user.getNick() + " 3 99 과도한 스크립트 사용";
			conn.doPrivmsg("^^", ban);
			LogUtil.fprintln(DateUtil.toTextString() +" " + ban);
			LogUtil.fprintln("Ban (AMsg) : " + user.getNick() + "/" + msg);
			scriptCount.remove(user.getNick());
			}
	}
	
	private void procNotice(String target, IRCUser user, String msg) {
		Integer count = noticeCount.get(user.getNick());
		if(count==null)
			count = 0;
		noticeCount.put(user.getNick(), ++count);
		
		if(count<1 + 1){
			conn.doPrivmsg(target, user.getNick() + "님 채널 노티스는 금지되어있습니다");
			LogUtil.fprintln("Warn(Notice) : " + user.getNick() + "/" + msg);
		}
		else{
			String ban = "ban " + target + " "  + user.getNick() + " 3 99 노티스금지";
			conn.doPrivmsg("^^", ban);
			LogUtil.fprintln(DateUtil.toTextString() +" " + ban);
			LogUtil.fprintln("Ban (Notice) : " + user.getNick() + "/" + msg);
			noticeCount.remove(user.getNick());
			}
		
	}
	
	
	
	
	private IRCEventListener listener = new IRCEventListener(){

		public void onDisconnected() {
			LogUtil.println("onDisconnected");
		}
		public void onError(String msg) {
			LogUtil.println("onError " + msg);
		}
		public void onError(int num, String msg) {
			LogUtil.println("onError " + num + msg);
		}
		public void onInvite(String chan, IRCUser user, String passiveNick) {
			LogUtil.println("onInvite " + user + " / " + passiveNick);	
		}
		public void onJoin(String chan, IRCUser user) {
			LogUtil.println("onJoin " + chan + " " + user);
		}
		public void onKick(String chan, IRCUser user, String passiveNick, String msg) {
			LogUtil.println("onKick " + chan + " " + user + " " + passiveNick + " " + msg);
		}
		public void onMode(String chan, IRCUser user, IRCModeParser modeParser) {
			LogUtil.println("onMode " + chan + " " + user + " " + modeParser.getLine());
		}
		public void onMode(IRCUser user, String passiveNick, String mode) {
			LogUtil.println("onMode " + user + " " + passiveNick + " " + mode);	
		}
		public void onNick(IRCUser user, String newNick) {
			LogUtil.println("onNick " + user + " " + newNick);
		}
		public void onNotice(String target, IRCUser user, String msg) {
			LogUtil.println("onNotice " + target + " " +  user + " " + msg);
			if( 
					target.equals("AUTH")|| user.getNick().equals("^^") || user.getNick().contains("hanirc.org")){
				
			}
			else if(user.getNick().toLowerCase().contains("hanirc")){
				
			}
			
			else
				procNotice(target, user, msg);
		}
		public void onPart(String chan, IRCUser user, String msg) {
			LogUtil.println("onPart " + chan + " " + user + " " + msg );
		}
		public void onPing(String ping) {
			LogUtil.println("onPing " + ping);
		}
		public void onPrivmsg(String target, IRCUser user, String msg) {
			LogUtil.println("onPrivmsg " + target + " " + user + " " + msg);	
			if(user.getNick().startsWith("kfmes"))
			{
				procCommand(target, msg);
			}
			if(msg.contains("쪽지") && msg.contains("확인") && 
					msg.toLowerCase().contains("alt") && msg.toLowerCase().contains("z")){
				String kick = "강퇴 " + target + " "  + user.getNick() + " 낚시하지 마세요";
				conn.doPrivmsg("^^",kick);
				LogUtil.fprintln(DateUtil.toTextString() +" " + kick);
				LogUtil.fprintln("Kick (Fishing) : " + user.getNick() + "/" + msg);
				
				//~~~입니다. 상태 : Tray 상태입니다. 현재 : 잠수중 입니다. 헤·즈 님의 호출횟수 
			}else if(msg.contains("상태 : ") && msg.contains("현재 : ")){
				procScriptCount(target, user, msg);
			}
			else if(msg.contains(""))
				procColorMessage(target, user, msg);
			if(msg.contains("ACTION"))
				procActionMessage(target, user, msg);
				
			
		}
		public void onQuit(IRCUser user, String msg) {
			LogUtil.println("onQuit " + user + " " + msg);
		}
		public void onRegistered() {
			LogUtil.println("onRegistered");
		}
		public void onReply(int num, String value, String msg) {
			LogUtil.println("onReply " + num + " " + value +" " + msg);
		}
		public void onTopic(String chan, IRCUser user, String topic) {
			LogUtil.println("onTopic " + chan + " " + user + " " + topic);	
		}
		public void unknown(String prefix, String command, String middle, String trailing) {
			LogUtil.println("unknown " + prefix + " " + command + " " + middle + " " + trailing);
		}
	};



	
}
