package klchateclipse;

import java.util.ArrayList;
import java.util.HashMap;

import klchateclipse.action.ConnectionAction;
import klchateclipse.action.DisconnectionAction;
import klchateclipse.action.UserInfoDialogOpenAction;

import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.PlatformUI;

public class ViewPart extends org.eclipse.ui.part.ViewPart {

	private Composite top = null;
	private CTabFolder cTabFolder = null;
	
	protected IRCClient irc = null;
	private ActionGroup action = null;
	
	private ConnectionAction connectionAction = null;
	private DisconnectionAction disconnectionAction = null;
	private UserInfoDialogOpenAction userInfoDialogOpenAction = null;
	
	protected Display display = null;  

//	private MenuBar menuBar = null;
	protected ArrayList<TabItem> tabList = new ArrayList<TabItem>();  //  @jve:decl-index=0:
	protected HashMap<String, TabItem> tabMap = new HashMap<String, TabItem>();
	private Text txt_input = null;

	@Override
	public void createPartControl(Composite parent) {
		init();
        GridData gridData1 = new GridData();
        gridData1.grabExcessHorizontalSpace = false;
        gridData1.verticalAlignment = GridData.CENTER;
        gridData1.horizontalAlignment = GridData.FILL;
        GridLayout gridLayout = new GridLayout();
        top = new Composite(parent, SWT.NONE);
        createCTabFolder();
        top.setLayout(gridLayout);
        txt_input = new Text(top, SWT.BORDER);
        txt_input.setLayoutData(gridData1);
        createCoolBar();
        

		txt_input.addKeyListener(new KeyListener(){

			public void keyPressed(KeyEvent e) {
				
			}

			public void keyReleased(KeyEvent e) {
				if(e.keyCode==13){
					System.out.println("key released");
					action.actionTyped(txt_input.getText().trim());
					txt_input.setText("");
				}
			}
		});
		
		initializeToolBar();
		
		
		

	}
	
	private void init(){
		irc = new IRCClient(this);
		action = new ActionGroup(this, irc);
	}

	@Override
	public void setFocus() {

	}
	

	private void createCTabFolder() {
		GridData gridData = new GridData();
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalAlignment = GridData.FILL;
		gridData.verticalAlignment = GridData.FILL;
		gridData.grabExcessVerticalSpace = true;
		cTabFolder = new CTabFolder(top, SWT.NONE);
		cTabFolder.setLayoutData(gridData);
		newTab("NETWORK");
		
	}
	
	public TabItem newTab(String chan) {
		CTabItem ti = new CTabItem(cTabFolder, SWT.CLOSE);
		Tab tab = new Tab(cTabFolder, SWT.BORDER);
		ti.setText(chan);
		ti.setControl(tab);
		TabItem ret = new TabItem(ti, tab);
		tabList.add(ret);
		tabMap.put(chan, ret);
		cTabFolder.setSelection(ti);
		return ret;
	}
	

	public String getShowingTab(){
		return cTabFolder.getSelection().getText();
	}
	public CTabFolder getCTabFolder(){
		return cTabFolder;
	}

	private void createCoolBar() {
		
	}
	
	private void initializeToolBar() {
		IToolBarManager toolbarManager = getViewSite().getActionBars()
		.getToolBarManager();

		makeAction();

		toolbarManager.add(connectionAction);
		toolbarManager.add(disconnectionAction);
		toolbarManager.add(userInfoDialogOpenAction);

	}
	
	private void makeAction(){
		IWorkbench workbench = PlatformUI.getWorkbench();
		ISharedImages platformImages = workbench.getSharedImages();
	      
		connectionAction = new ConnectionAction(irc);
		connectionAction.setImageDescriptor(platformImages
		         .getImageDescriptor(ISharedImages.IMG_TOOL_UP));
		connectionAction.setToolTipText("Connection Action");
		
		disconnectionAction = new DisconnectionAction(irc);
		disconnectionAction.setImageDescriptor(platformImages
		         .getImageDescriptor(ISharedImages.IMG_TOOL_DELETE));
		disconnectionAction.setToolTipText("Disconnection Action");
		
		userInfoDialogOpenAction = new UserInfoDialogOpenAction(getSite().getShell());
		userInfoDialogOpenAction.setImageDescriptor(platformImages
		         .getImageDescriptor(ISharedImages.IMG_TOOL_DELETE));
		userInfoDialogOpenAction.setToolTipText("Dialog Open Action");
		
	}
	
	
	public static ImageDescriptor getImage(String str){
		ImageDescriptor descriptor = null;
		descriptor = Activator.getImageDescriptor("icons/"+str);	
		
		return descriptor;

	}


}
